/************************************************************************************
 * Langtons Ameise
 * (C) Copyright 2002, Heiko Khler
 *
 * Das Spiel erfand der Informatiker Christopher Langton (Langtons Ameise).
 * Das Spiel insgesamt stellt einen zellurren Automaten dar. Dabei handelt
 * es sich um ein Gitternetz von Schaltern, die unterschiedlich eingestellt
 * werden knnen. Entscheidend sind aber die Regeln fr die Bettigung der
 * Schalter, also wie sich die Ameise zu bewegen hat und wie sich die Farben
 * der einzelnen Zellen dabei ndern:
 * 
 *        -Ist sie auf einer grauen Zelle, macht sie eine 90-Wendung nach
 *         links.
 *        -Ist sie auf einer weien Zelle, macht sie eine 90-Wendung nach
 *         rechts.
 *        -Beim bergang zur nchsten Zelle ndert diejenige, auf der sie
 *         gerade ist, ihre Farbe von Wei in Schwarz oder umgekehrt.
 *
 * Langtons Ameise wurde verffentlicht in dem Buch "Chaos und Anti-Chaos
 * - Ein Ausblick auf die Wissenschaft des 21. Jahrhunderts" von
 * Jack Cohen und Ian Stewart (S. 274 - 276). Es erschien im Mai 1997 bei:
 * Deutscher Taschenbuch Verlag GmbH. & Co. KG , Mnchen.
 ************************************************************************************/

#include <time.h>
#include <graphics.h>
#include "main.h"


void draw_field( bool field[FIELD_WIDTH][FIELD_HEIGHT] )
{
	for(int x = 0; x < FIELD_WIDTH; x++)
		for(int y = 0; y < FIELD_HEIGHT; y++){
			if( field[x][y] )
				setfillstyle(SOLID_FILL, BLACK);
			else
				setfillstyle(SOLID_FILL, WHITE);
			bar(x*SQUARE_WIDTH, y*SQUARE_WIDTH, (x+1)*SQUARE_WIDTH, (y+1)*SQUARE_WIDTH);
		}
}

void init_field( bool field[FIELD_WIDTH][FIELD_HEIGHT] )
{
	for( int x = 0; x < FIELD_WIDTH; x++)
		for(int y = 0; y < FIELD_HEIGHT; y++)
			if( x >= RECT_START_WIDTH  && x <= RECT_END_WIDTH &&
				y >= RECT_START_HEIGHT && y <= RECT_END_HEIGHT )
					field[x][y] = true;
				else
					field[x][y] = false;
}

void redraw_square( bool field[FIELD_WIDTH][FIELD_HEIGHT], Pos pos, bool current_pos )
{
	if(current_pos)
		setfillstyle(SOLID_FILL, RED);
	else if( field[pos.x][pos.y] )
		setfillstyle(SOLID_FILL, BLACK);
	else
		setfillstyle(SOLID_FILL, WHITE);
	bar(pos.x*SQUARE_WIDTH, pos.y*SQUARE_WIDTH, (pos.x+1)*SQUARE_WIDTH, (pos.y+1)*SQUARE_WIDTH);
}

void run_ant( bool field[FIELD_WIDTH][FIELD_HEIGHT], Pos pos )
{
	Direction dir = LEFT;
	clock_t _clock;
	Pos prev_pos;
	unsigned long _time = 0;
	_clock = clock();
	// ermittel Gre von _time
	while( clock()-_clock < CLOCKS_PER_SEC/TIME_SLICE )
		_time++;
	for( int t = 0; t < STEPS; t++ ){
		// ist Ameise auf schwarzer Zelle 90 Wendung nach links
		// ist Ameise auf weier Zelle 90 Wendung nach rechts
		for(int i = 0; i < _time; i++)
			;
		switch( dir){
		case UP:
			if( field[pos.x][pos.y] )
				dir = LEFT;
			else
				dir = RIGHT;
			break;
		case LEFT:
			if( field[pos.x][pos.y] )
				dir = DOWN;
			else
				dir = UP;
			break;
		case DOWN:
			if( field[pos.x][pos.y] )
				dir = RIGHT;
			else
				dir = LEFT;
			break;
		case RIGHT:
			if( field[pos.x][pos.y] )
				dir = UP;
			else
				dir = DOWN;
			break;
		}
		// dreht Ameise
		switch( dir ){
		case UP:
			pos.y--;
			break;
		case LEFT:
			pos.x--;
			break;
		case DOWN:
			pos.y++;
			break;
		case RIGHT:
			pos.x++;
			break;
		}
		// ndert Farbe von aktueller Zelle
		if( field[pos.x][pos.y] )
			field[pos.x][pos.y] = false;
		else
			field[pos.x][pos.y] = true;
		redraw_square(field, prev_pos, false);
		prev_pos = pos;
		redraw_square(field, pos, true);
	}
}

int main( int argc, char *argv[])
{	
	bool field[FIELD_WIDTH][FIELD_HEIGHT];
	Pos start_pos;

	start_pos.x = START_POS_X;
	start_pos.y = START_POS_Y;
	init_field(field);
	initgraph(FIELD_WIDTH*SQUARE_WIDTH, FIELD_HEIGHT*SQUARE_WIDTH);
	draw_field(field);
	run_ant( field, start_pos );
	getch();
	closegraph();
	return 0;
}